!
! Copyright (C) 2001-2003 PWSCF group
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
! Eyvaz Isaev, Condensed Matter Theory Group, Uppsala University and
!              Theoretical Physics Department, Moscow State Institute of Steel and Alloys
! E-mail: Eyvaz.Isaev@fysik.uu.se, eyvaz_isaev@yahoo.com
!
!     Debye temperature calculation at various T
!
! Some definitions of former fqha.f90 were used
!
program Debye
  !
!  implicit none
  implicit real*8(a-h,o-z)
  integer, parameter:: ndivx=10000
  real(kind=8) :: dos(ndivx),nu(ndivx) ,T,a1,a2,a3,a4
  real(kind=8) :: de, emax 
  real(kind=8) :: CV, CV_tot, q1,q2,q3
  integer :: i,ndiv, n1
  integer :: natoms
  character(len=80) :: filename
  real(kind=8) :: pdos(ndivx),gx(ndivx),gy(ndivx),gz(ndivx)
  character(len=1) :: dummy
!  
!
  a1=0.5d0/13.6058d0/8065.5d0
  a2=8.617d-5/13.6058d0
  a3=1.0d0/8065.5d0/8.617d-5

! K_B
   a4=1.d0/13.6058d0/8065.5d0
    read(5,*)
  read (5,*) dummy, ndiv, emax, de
  if (ndiv.gt.ndivx) stop ' ndivx too small'


!  do i=1,ndiv-1
!     read(5,*) nu(i),dos(i),pdos(i),gx(i),gy(i),gz(i)
!  enddo
!
! More safe reading a file 
        i=1
100     read(5,*,end=99) nu(i),dos(i),pdos(i),gx(i),gy(i),gz(i)
        i=i+1
        goto 100
99      continue

    ndiv=i

     open(19,file='debye.in')
     read(19,*) natoms
     read(19,*) T
     close(19)
     
! Restriction on T
    if(T.le.4) then
    write(6,'("# Temperature below 4K is not recommended for the Debye temperature calculations")') 
    stop
    endif

! The heat capacity in a standard way
!
    CV=0.

    do i=2,ndiv-3,3

     q1=0.5*a3*nu(i)/T
     q2=0.5*a3*nu(i+1)/T
     q3=0.5*a3*nu(i+2)/T
    
     CV = CV + 3*dos(i)*q1*q1/sinh(q1)**2 + &
   &           3*dos(i+1)*q2*q2/sinh(q2)**2 + &   	
   &           2*dos(i+2)*q3*q3/sinh(q3)**2 

   enddo 

    CV_tot=CV*de*3/8


! Find the Temperature range [A:B] with opposite F(A) and F(B)
!
 	A=1
	B=10
	N=1000
      zero=0.0
      FACT=2
11    F1=CV_tot-Debye_int(natoms,T,A)
      F2=CV_tot-Debye_int(natoms,T,B)
      DO 31 I=1,N
      IF(F1*F2.LT.zero) GO TO 51
      IF(ABS(F1).LT.ABS(F2)) GO TO 21
      B=B+FACT*(B-A)
      F2=CV_tot-Debye_int(natoms,T,B)
      GO TO 31
   21 A=A+FACT*(A-B)
      F1=CV_tot-Debye_int(natoms,T,A)
   31 CONTINUE
   51 continue

      F1=CV_tot-Debye_int(natoms,T,A)
      F2=CV_tot-Debye_int(natoms,T,B)

	itmax=1000000
	eps=0.001
!	zero=0.d0
	zero=1.d-6
	
! Now try to find the Debye temperature
!
      FL=F1 ! F(A)
      FR=F2 ! F(B)

      IF(FL*FR.LE.ZERO) GO TO 10
      GO TO 70
   10 IF(FL.GE.ZERO) GO TO 20
      XL=A
      XR=B
      GO TO 30
   20 XL=B
      XR=A
      temp=FL
      FL=FR
      FR=temp
   30 DX=XR-XL
      DO 60 J=1,ITMAX
      if(J.eq.Itmax) stop 'Maximum iterations number exceeded'
      ROOT=XL+DX*FL/(FL-FR+1d-12)
      FR=CV_tot - Debye_int(natoms,T,ROOT)
      IF(FR.GE.zero) GO TO 40
      DEL=XL-ROOT
      XL=ROOT
      FL=FR
      GO TO 50
   40 DEL=XR-ROOT
      XR=ROOT
      FR=FR
   50 DX=XR-XL
      IF(dabs(del).LT.dabs(eps).OR.FR.EQ.zero) GO TO 80
   60 CONTINUE
70    stop 'Debye program stopped, no appropriate interval with FL*FR<0'   
80    continue

     write(6,'(f8.2, 3x,3f12.6)') T, CV_tot, dabs(root), Debye_int(natoms,T,ROOT)
!     write(6,'("CV==",f12.6, 3x,"T_D==",f12.6)') CV_tot, Debye_int(natoms,T,ROOT)
     
  stop 
end program Debye

